;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Loading this file installs CG/JS.
;; Please see readme.txt for more information
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(in-package :user)

(eval-when (compile eval load)
  (require :shell)
  (use-package :excl.shell))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; validate files

(let ((platform-file
       (directory
	(merge-pathnames
	 "PLATFORM-*"
	 (path-pathname *load-pathname*)))))
  (or platform-file
      (error "Could not find the PLATFORM file."))
  
  (when (> (length platform-file) 1)
    (error "There is more than one PLATFORM file: ~a." platform-file))
  (setq platform-file (car platform-file))
  
  (or (=~ "^PLATFORM-(.*)" (file-namestring platform-file))
      (error "Cannot parse the PLATFORM file."))
  
  (let ((expected-platform $1)
	(current-platform
	 (format nil "~a~a" (symbol-name sys::.os.) #+smp "smp" #-smp "")))
    (or (string-equal expected-platform current-platform)
	(error
	 "The current platform (~a) does not match the downloaded version (~a)."
	 current-platform expected-platform)))
  ;; We are on the correct platform, fall through...
  )

;; HTML files needed for the operation of CG/JS
;; (this variable's value is generated by build_cgjs.sh)
(defvar *html-files* '("cgjs.html" "ssl-not-found.html" "browser-not-supported.html" ))

(defvar *extra-files*)
(setq *extra-files* nil)

(dolist (file *html-files*)
  (setq file (merge-pathnames file *load-pathname*))
  (or (probe-file file)
      (error "~a does not exist." file))
  (push file *extra-files*))
;; All *html-files* exist...

;; These two are directories but the pathname is specified as
;; a file, so merging works below.
(defvar *cg2js-directory* (merge-pathnames "code.cg2js" *load-pathname*))
(defvar *idejs-directory* (merge-pathnames "code.idejs" *load-pathname*))

(dolist (d (list *cg2js-directory* *idejs-directory*))
  (or (probe-file d)
      (error "~a does not exist." d)))

;; MD5 checksums of the incoming files
;; (this variable's value is generated by build_cgjs.sh)
(defvar *checksums* 
'(
(#xe5e47a10fb9f332666041d4d81f9e503 . "./code.cg2js/cg-status-bar.fasl")
(#x62d5214317b3e35169b08e5730b4188e . "./code.cg2js/cg-hotspot.fasl")
(#xe3f909b8f1beca0b495a520339be4273 . "./code.cg2js/cg-toolbar.fasl")
(#x86072dd629cdd9f1171722489e0013be . "./code.cg2js/cg-clipboard-stack.fasl")
(#xa1a0e560e33bb11d2950f9740298efc4 . "./code.cg2js/cg-tooltip.fasl")
(#x4f5d4bf56ebe18ec84822e7886d72c0b . "./code.cg2js/cg-calendar.fasl")
(#x8d51d71ea05c4b1aa8f77a9a3d7a4e6c . "./code.cg2js/cg-item-list.fasl")
(#xbbbe4b163a2677b4eb0773d8c48f7418 . "./code.cg2js/cg-bitmap-stream.fasl")
(#xc0c3991e40dc32ad4537c1d18c2c0cff . "./code.cg2js/cg-trackbar.fasl")
(#x5a22c8df38f9f9d853db53461e246904 . "./code.cg2js/cg-class-grid.fasl")
(#xdf95dd09259cf0cb1b5701248f1991b0 . "./code.cg2js/cg-dialog-item.fasl")
(#x4d6e18dfbcb59220e7f1dc1080c0ad56 . "./code.cg2js/cg-comtab.fasl")
(#x4038dd8cfa14016a7fb59bf904e1f748 . "./code.cg2js/cg-lisp-text.fasl")
(#xffc3919a59101b0ec68f26c04dcd301d . "./code.cg2js/cg-button.fasl")
(#x0046651267cc6d86393638fb56e9bd0c . "./code.cg2js/cg-printing.fasl")
(#xab5ea881e8f0670415f6ada30e655f32 . "./code.cg2js/cg-pixmap-printing.fasl")
(#x55507d17898f6a668fd7d3190711ff33 . "./code.cg2js/cg-multi-picture-button-tooltip.fasl")
(#xf3ae69ea6e2f957cd0687c8ffede86ac . "./code.cg2js/cg-pixmap-widget.fasl")
(#x34c7de1422bc20202d564f56bd6f2940 . "./code.cg2js/cg-selected-object.fasl")
(#x329ba67a03515a0ef81ecf12a50eee90 . "./code.cg2js/cg-text-or-combo.fasl")
(#x6b5703d046027fb9f2b2d2295dfe4ed4 . "./code.cg2js/cg-graphics-context.fasl")
(#x8788b8834f87256f9deaf4fdce950001 . "./code.cg2js/cg-edit-in-place.fasl")
(#x8bcd4ff24c1f246c50bcac2931791594 . "./code.cg2js/cg-string-dialog.fasl")
(#x9369ba7d618a090b60aaf4e2bc996388 . "./code.cg2js/cg-list-view.fasl")
(#xc5c456aa1f62ae03f51da14367601347 . "./code.cg2js/cg-intersect.fasl")
(#x11e4e4f2ba6fc20ea5c295a544b3915c . "./code.cg2js/cg-text-widget.fasl")
(#x989f721e1afbd7845dda251e3c4116fd . "./code.cg2js/cg-directory-dialog.fasl")
(#x13170df43601e8b7df4440542594dbee . "./code.cg2js/cg-file-dialog.fasl")
(#x2e6a6d322fcf5e15dbdeeb058af5ebda . "./code.cg2js/cg-dde-utils.fasl")
(#x432c431c1ad6e2ea944084396abd73f9 . "./code.cg2js/cg-nodes.fasl")
(#xf4a9a4696cd595b86fb7e924f4469050 . "./code.cg2js/cg-text-edit-pane-mark.fasl")
(#x70e70eadae2f230e7399ddefd46bb735 . "./code.cg2js/cg-up-down-control.fasl")
(#x1fd8b3a6ea76191bec7507b5954c301b . "./code.cg2js/cg-get-position.fasl")
(#x807d2528265f09cfbf1adf4a812b792d . "./code.cg2js/cg-toggling-widget.fasl")
(#xf0ea5bc5abffa71193180d48ea0364dc . "./code.cg2js/cg-acache.fasl")
(#x5080df6ff516ebb7837052ad9e0ad91c . "./code.cg2js/cg-outline-drag-and-drop.fasl")
(#x178dcfdf66aacddb145b229030f85655 . "./code.cg2js/cg-base.fasl")
(#x3a0b2f43cae7b2d687874cfe1839f99b . "./code.cg2js/cg-drag-and-drop.fasl")
(#x0f990d4735eac68bfd0e013f61649d37 . "./code.cg2js/cg-find-string-dialog.fasl")
(#x51feb61ae7c5e418d2b5e6addcc4eb99 . "./code.cg2js/cg-property.fasl")
(#xd32b86fc4f4e47b7ed9b086e645bf35b . "./code.cg2js/cg-editable-text.fasl")
(#x2a40bbd2df56768222a63b3a510d7443 . "./code.cg2js/cg-lamp.fasl")
(#x5a7d4fee41ed7a8b0d89aaa14b0fae3c . "./code.cg2js/cg-progress-indicator.fasl")
(#xd47138eba0c032e5751754e66ff60763 . "./code.cg2js/cg-chart-widget.fasl")
(#xa67c862e0f0d8c48e077602f3544e323 . "./code.cg2js/cg-common-control.fasl")
(#x3cb96cbff6bf81289c1e27961792df62 . "./code.cg2js/cg-font-dialog.fasl")
(#xd3c845c7fb2b034a635b4d6bdfd5b61c . "./code.cg2js/cg-gesture-emulation.fasl")
(#xfa18dc48510246282b30d393587e121e . "./code.cg2js/cg-pixmap-file-io.fasl")
(#x4b2fc673eab0435b3dc1eb2b0ec1b9db . "./code.cg2js/cg-os-window.fasl")
(#x4d9ab21c286189029d8c2f17474ca356 . "./code.cg2js/cg-multi-line-lisp-text.fasl")
(#xccacd3be35489c950d91144f0196144d . "./code.cg2js/cg-pixmap.fasl")
(#x3d56aa98317a3675e49c77fc0a53f697 . "./code.cg2js/cg-chart-or-plot.fasl")
(#x97d00f1936b8b40126451c5913931909 . "./code.cg2js/cg-scroll-bar-mixin.fasl")
(#x38e1dd61383d41b76ca5eeb7a3015340 . "./code.cg2js/cg-scaling-stream.fasl")
(#x53b5e0a268ab2c33d652ff8f58db2d65 . "./code.cg2js/cg-grid-widget-drag-and-drop.fasl")
(#xdb7673b6633d5682a8d37d74c2f0cf05 . "./code.cg2js/cg-picture-widget-palette.fasl")
(#xd0574fa31e4ec7da59b09063a5da7869 . "./code.cg2js/cg-timer.fasl")
(#x482034343faf5ae597082cf2c4d0967b . "./code.cg2js/cg-scroll-bar.fasl")
(#x5b9ab793e025c92d22c7dfb5304d2114 . "./code.cg2js/cg-caret.fasl")
(#x325dc43901eee0ede77858adc690e6d9 . "./code.cg2js/cg-template-string.fasl")
(#xfc8f5dd3fe7d1c61dce1585433d4d7c4 . "./code.cg2js/cg-multi-picture-button-drag-and-drop.fasl")
(#x4d1bae1d1d0be907d258da4e91091af4 . "./code.cg2js/cg-directory-dialog-os.fasl")
(#xfbf2d27862e5d20d03c123572d82efb3 . "./code.cg2js/cg-combo-box.fasl")
(#x997c7da33e44716337c1c299c335246c . "./code.cg2js/cg-text-edit-pane.fasl")
(#x64d74ec5a6b4fbc9cfe1d1a57e8f9a3b . "./code.cg2js/cg-multi-picture-button.fasl")
(#x0745f7ce11399ff7aa0c9d310e6dc257 . "./code.cg2js/cg-menu-tooltip.fasl")
(#x9a2f882990dc7def5408b18d5090247e . "./code.cg2js/cg-scrolling-static-text.fasl")
(#x033af2f2249dc6338355eeb48fe04808 . "./code.cg2js/cg-player.fasl")
(#xc8a16fd4cdf2d16963065548cdf11e03 . "./code.cg2js/cg-outline.fasl")
(#xbb9c4e9b582ae47b8169a3d6c12d2d1c . "./code.cg2js/cg-class-support.fasl")
(#xf27de512824e948c0812309d3512355d . "./code.cg2js/cg-lettered-menu.fasl")
(#xb73e650bab77c36cef50e0403f09795e . "./code.cg2js/cg-cursor-pixmap.fasl")
(#xdde19e97946641135dd51b3ef3b94803 . "./code.cg2js/cg-fill-texture.fasl")
(#x82835ed21e6f69e64785d670aa8f9cb6 . "./code.cg2js/cg-lisp-widget.fasl")
(#xc784f9dafc0fbade9841887f48a2b75b . "./code.cg2js/cg-keyboard-shortcuts.fasl")
(#x1f8897df18451a2774eb0cd10e387dcf . "./code.cg2js/cg-clipboard.fasl")
(#x1536a8fa8dd72f7a848a6a638e33b7f5 . "./code.cg2js/cg-wrap-string.fasl")
(#xa592bf5d721398c70c07236a683dc873 . "./code.cg2js/cg-os-widget.fasl")
(#x8162824b8de29066d801ac7e47d87a55 . "./code.cg2js/cg-check-box.fasl")
(#x925a6ba180dfb941c909f7371879c011 . "./code.cg2js/cg-choice-list.fasl")
(#x8e6b8ca09f637dcf8f7dd6092ca852c6 . "./code.cg2js/cg-layout.fasl")
(#xf4b64d0fdadf913fb7adcf6883985c35 . "./code.cg2js/cg-group-box.fasl")
(#x133a09f5e5dcc2f302d8cf7f7b1106cb . "./code.cg2js/cg-bitmap-pane-clipboard.fasl")
(#xfe107c1cbc0494065618b834def201b6 . "./code.cg2js/cg-radio-button.fasl")
(#x57e05dd9f0501b9e9a385513cac2e8de . "./code.cg2js/cg-tab-control.fasl")
(#xa802a9e3763297a4fa2ac1180a9bce8c . "./code.cg2js/cg.fasl")
(#x896f4bdc79ed72a832362edf3b0aa786 . "./code.cg2js/cg-drawable-clipboard.fasl")
(#xc486e03bacdbc00d1f226f65318941b6 . "./code.cg2js/cg-dropping-outline.fasl")
(#xc2aa5d70cf5d6b77166028ee16448d93 . "./code.cg2js/cg-get-pixmap.fasl")
(#x29c9243be5aebab9a303ae7a1d38edee . "./code.cg2js/cg-grid-widget.fasl")
(#x58cab8198bcddf4cc5b4eaa9b8e18d60 . "./code.cg2js/cg-utility-dialog.fasl")
(#x23d7ffd84d99d1f1f1774ce3e11f7155 . "./code.cg2js/cg-static-text.fasl")
(#xe84babc2829e04d70ebef986fd202d77 . "./code.cg2js/cg-class-slot-grid.fasl")
(#x337ce279d9624be5b1ad5dc5bac18519 . "./code.cg2js/cg-object-editor-layout.fasl")
(#x150c8b47cd70d48456cbf3e17163d82a . "./code.cg2js/cg-drag-and-drop-image.fasl")
(#x122563031c41d5e585d280d80ccef9a9 . "./code.cg2js/cg-bitmap-pane.fasl")
(#x767b37a09b1015981910bc9144e51309 . "./code.cg2js/cg-plot-widget.fasl")
(#xfbea9c85aa5beb4dfdbc74197b7c07dd . "./code.cg2js/cg-color-dialog.fasl")
(#xe751acf8a6dce85e4020ec72a2d0d6ab . "./code.cg2js/cg-icon-pixmap.fasl")
(#xa523eed706685ffa768187169461a418 . "./code.cg2js/cg-icon.fasl")
(#xe62057329e138cc01286473cafdb9aea . "./code.cg2js/cg-dev.fasl")
(#x5f703989f10b09c6822ec63651c1cc78 . "./code.cg2js/cg-choose-printer.fasl")
(#x790fe3531d9488b2e1788611644c4faa . "./code.cg2js/cg-intersect-posbox.fasl")
(#x253027801be832114c41cb4c8a1289b1 . "./code.cg2js/cg-menu.fasl")
(#x75f1a83475b49acba3ddf5eba2de7515 . "./code.cg2js/cg-yes-no-list.fasl")
(#xbce803bd8e5919646fb1548ab0197ca8 . "./code.cg2js/cg-palette.fasl")
(#x170f7c5d9e01597be5c8051cef4f6bd6 . "./code.cg2js/cg-outline-edit-in-place.fasl")
(#x5f8a2c7dcc3d854f2cffe2d4f3f3dc7d . "./code.cg2js/cg-message-dialog.fasl")
(#xc5ceed3ad3e833990988c3719e58c142 . "./code.cg2js/cg-shortcut-menu.fasl")
(#x3e3754e36097bc26d22e13c34e1a49a9 . "./code.cg2js/cg-split-bar.fasl")
(#xe991373d65b40eccbf5724f8eb80e8bf . "./code.cg2js/cg-multi-line-editable-text.fasl")
(#xd232cad1f8f0c4a70bf916e32051593b . "./code.cg2js/cg-web-browser.fasl")
(#x113e8987615d7e361310636daf8c893d . "./code.cg2js/cg-project-window.fasl")
(#xaa93a24c86e6a2ee5e99d8be65a443bf . "./code.cg2js/cg-picture-widget.fasl")
(#x069fbfcb10a7b24b850f9aa112f7e45a . "./code.cg2js/cg-pixmap-rotate.fasl")
(#x93d838dae0dc224298755b53254dc41e . "./code.cg2js/cg-object-editor.fasl")
(#x5eab595e1b0311ac83746daee96298ad . "./code.cg2js/cg-paren-matching.fasl")
(#x4ac6a83cba52661bc309edf7d812be25 . "./code.cg2js/cg-sample-file-menu.fasl")
(#xa8ba6139c3b9ce23cb3014d7cd6530b7 . "./code.cg2js/cg-drawable.fasl")
(#x0124db60475c9d4491391630241b176c . "./code.cg2js/cg-text-edit-pane-file-io.fasl")
(#x14d8c74d29033d87ba4d8b93478d4bec . "./code.cg2js/cg-html-widget.fasl")
(#xff1e30478f3d4d32d21bc80d78850a3b . "./code.cg2js/cg-curve.fasl")
(#x31c68a04980465eedcc29e6585bfaaae . "./code.cg2js/cg-yes-no-string.fasl")
(#xf238427079a7b9ae15b8cecb17f489e8 . "./code.cg2js/cg-lisp-edit-pane.fasl")
(#xa6f9768cf3b98624c7a2603bac857c24 . "./code.cg2js/cg-html-dialog.fasl")
(#x1668d86e81f12a236a378a3c5b83314a . "./code.cg2js/cg-clipboard-pixmap.fasl")
(#x81602f90fbebada5d523403c381760e0 . "./code.cg2js/cg-tray.fasl")
(#xb0a4ee6143751caa5a9c74e8271ba77e . "./code.idejs/ide-apropos.fasl")
(#x6efc727754cc9c53f616f38d3629b137 . "./code.idejs/ide-window-list.fasl")
(#xc5cc21f6ead47209be7ebce85fa0a91f . "./code.idejs/ide-class-browser.fasl")
(#xb0cab11afe17939fb162e9fec6937534 . "./code.idejs/ide-base.fasl")
(#x274bdbf6c4aada43bf8d114d5e2ed5c2 . "./code.idejs/ide-process-browser.fasl")
(#xddfd2a35788006c72b9eaea8049d7821 . "./code.idejs/ide-acache.fasl")
(#x63621ec1caf367851988ca654f2e6a10 . "./code.idejs/ide-navigator.fasl")
(#x847598b53decf5bccd9c2e72227b9e11 . "./code.idejs/ide-trace.fasl")
(#xfd2e101a0620e0551aff3833730fb026 . "./code.idejs/ide-project.fasl")
(#x4e38b5c2efc15994d585934b91673b9d . "./code.idejs/ide-patch-dialog.fasl")
(#x853f2c010e277ab1cd83c38574156904 . "./code.idejs/ide-grapher.fasl")
(#x74aa3603627697e8c3c2f620ce037f47 . "./code.idejs/ide-dev.fasl")
(#xdfdd5d04ff3414352d07928822bb956e . "./code.idejs/ide-keyshort.fasl")
(#xe227ac16bcdeec8dac6d391ba0ddf8ae . "./code.idejs/ide-editor.fasl")
(#xc385bf4a07b10fae2be4f675b29946e1 . "./code.idejs/ide-inspector.fasl")
(#x014ac081f294518207f30242afb20fad . "./code.idejs/ide.fasl")
(#xffe8a8031c13a0c52a3c62d8eef93de1 . "./code.idejs/ide-clipboard.fasl")
(#x3da14573ef436a9d2e75987c4c388c4b . "./code.idejs/ide-find-definitions.fasl")
(#xfa347ef44dcc8c768c151d1ae69de430 . "./code.idejs/ide-form.fasl")
(#x96a6b7be2ccc1cd9f3db085e601b2fc1 . "./code.idejs/ide-intro.fasl")
(#x44c6a10dfb3add004417d67b4f31a1a3 . "./code.idejs/ide-find-in-files.fasl")
(#x6e989d8304a2a18bd132b9293bb21ee7 . "./code.idejs/ide-package-list.fasl")
(#x054645851168937012df0ebdb8416de7 . "./code.idejs/ide-class-grapher.fasl")
(#xf3f59eb0bf604c0b34476c9897472f9c . "./cgjs.html")
(#xdac1739373835d1a63cbafa2c3e0b06a . "./ssl-not-found.html")
(#x90c6d2a65504d45266ad9fa3a49be2f9 . "./browser-not-supported.html"))
 )

(let ((files-ok
       (dolist (checksum *checksums* t)
	 (when (/= (car checksum)
		   (md5-file
		    (merge-pathnames (cdr checksum) *load-pathname*)))
	   (return nil)))))
  (cond
   ((not files-ok)
    (error "The extracted files failed the checksum test.  ~
Please download and/or extract again."))
   (t
    (format t "Checksums on files are OK...~%"))))

;; We have validated all the files are there...

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; check for needed patches

(eval-when (compile eval load) (require :build))

(defvar *required-patches* '(#+macosx ("build" . 3)))

(dolist (required-patch *required-patches*)
  (let ((existing-patch (assoc (car required-patch)
			    (cdr (assoc :system sys::*patches*))
			    :test #'string=)))
    (when (null existing-patch)
      (error "Could not find patch info for the ~s module."
	     (car required-patch)))
    (when (not (>= (second existing-patch) (cdr required-patch)))
      ;; The build patch number is NOT >= to the one we require
      (error "~
Expected version ~d of ~s but have version ~d.  Please update ~
your lisp with update.~a and then do the CG/JS installation again."
	     (cdr required-patch) (car required-patch)
	     (second existing-patch)
	     #+mswindows "exe"
	     #-mswindows "sh"))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; copy files

(defun copy (source dest)
  (cond
   ((file-directory-p source)
    (format t "Copy directory ~a~%" (enough-namestring source))
    (copy-directory source dest :quiet t))
   (t
    (format t "Copy file ~a~%" (enough-namestring source))
    (sys:copy-file source dest :verbose nil))))

(defvar *sys* (translate-logical-pathname (pathname "sys:")))

(cond
 ((excl::pathname-equalp (truename *sys*)
			 (path-pathname *load-truename*))
  (format t "Files are already in place, no copying requiring.~%"))
 (t ;; copy files, replacing them with new ones if they already exist
  (let (dest)
    (dolist (from *extra-files*)
      (setq dest (merge-pathnames (file-namestring from) *sys*))
      (when (probe-file dest) (delete-file dest))
      (copy from dest))
    
    (dolist (from (list *cg2js-directory* *idejs-directory*))
      (setq dest (merge-pathnames (file-namestring from) *sys*))
      ;; now make it a directory pathname
      (setq dest (pathname-as-directory dest))

      (when (probe-file dest) (delete-directory-and-files dest))
      (copy (pathname-as-directory from) dest)))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; update develenv.cl to load CG/JS before the original version of CG

(defvar *setup-cgjs-loading* "sys:setup-cgjs-loading.cl")

;; Because users might use an ANSI lisp to install, have to make sure
;; the resulting file can be loaded into an ANSI and Modern lisp.
(with-open-file (s *setup-cgjs-loading* :direction :output
		 :if-exists :supersede)
  (format s "~
;; This file was created by install-cgjs.cl.
;; Please do not edit it.

")
  (dolist (dir (list "code.cg2js" "code.idejs"))
    (format s "
(let ((path (make-pathname
             :host \"sys\"
             :directory '(:relative \"~a\")
	     :type \"fasl\"
	     )))
  (format t \";; *require-search-list*: adding: ~~s~%\" path)
  (push path sys:*require-search-list*))

"
	    dir)))

(defvar *develenv.cl* (pathname "sys:develenv.cl"))

(when (not (probe-file *develenv.cl*))
  (setf (file-contents *develenv.cl*)
    ";; This file is loaded before images are rebuilt with update.sh/update.exe

"))

(let ((contents (file-contents *develenv.cl*)))
  (cond
   ((match-re
     (format nil "^#\\|\\|# \\(load \"sys:~a"
	     (file-namestring *setup-cgjs-loading*))
     contents
     :multiple-lines t)
    (format t "develenv.cl already correctly loads CG/JS.~%"))
   ((match-re
     (format nil "^#\\+ignore \\(load \"sys:~a"
	     (file-namestring *setup-cgjs-loading*))
     contents
     :multiple-lines t)
    (format t "develenv.cl: CG/JS disabled; will enable.~%")
    (setf (file-contents *develenv.cl*)
      (replace-re  contents
		  (format nil "^#\\+ignore *\\(load \"sys:~a"
			  (file-namestring *setup-cgjs-loading*))
		  (format nil "#||# (load \"sys:~a"
			  (file-namestring *setup-cgjs-loading*))
		  :multiple-lines t
		  :count 1)))
   (t
    (format t "develenv.cl: adding CG/JS initialization.~%")
    (setf (file-contents *develenv.cl*)
      (concatenate 'simple-string
	contents
	(format nil "~

;; To disable the use of CG/JS and go back to the originally released
;; (and possibly patched) version of CG/IDE, change the #||# to
;; #+ignore on the following line and the rebuild images by running
;; update.sh or update.exe, depending on your platform:
#||# (load \"sys:~a\")~%~%"
		(file-namestring *setup-cgjs-loading*)))))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; success!

(let ((msg (format nil "~
CG/JS installation successful!

NOTE: to use CG/JS you must rebuild images by running 'update.~a'
      in the Allegro CL directory (~a)."
		   #+mswindows "exe"
		   #-mswindows "sh"
		   *sys*)))
  #-mswindows (format t "~&~%~a~%" msg)
  #+mswindows (excl::internal-message-box msg "CG/JS Installation" :type :ok))
